/**
 * ZRE - A lightweight, portable programming language for permacomputing.
 * Copyright (C) 2025 zongor
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
#include "parser.h"
#include "compiler.h"
#include "debug.h"
#include "vm.h"
#ifdef __EMSCRIPTEN__
#include <emscripten.h>
#endif

uint32_t pc = 1; /* Program counter */

Code *code;

void mainloop() {
  pc = step_vm(code->memory, code->size, pc);
  if (pc == 0) {
#ifdef __EMSCRIPTEN__
    emscripten_cancel_main_loop(); /* this should "kill" the app. */
#else
    core_dump(code->memory, code->size);
    exit(0);
#endif
  }
}

int main(int argc, char **argv) {
  code = demo_add_compile();

#ifdef __EMSCRIPTEN__
  emscripten_set_main_loop(mainloop, 0, 1);
#else
  while (1) {
    mainloop();
  }
#endif
}
