/**  
 * ZRE - A lightweight, portable programming language for permacomputing.  
 * Copyright (C) 2025 zongor
 *  
 * This program is free software: you can redistribute it and/or modify  
 * it under the terms of the GNU General Public License as published by  
 * the Free Software Foundation, either version 3 of the License, or  
 * (at your option) any later version.  
 *  
 * This program is distributed in the hope that it will be useful,  
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 * GNU General Public License for more details.  
 *  
 * You should have received a copy of the GNU General Public License  
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.  
 */  
#ifndef ZRE_PARSER_H
#define ZRE_PARSER_H

#include "common.h"
#include <string.h>

typedef enum TokenType
{
  /* Single char tokens */
  TOKEN_LEFT_PAREN,
  TOKEN_RIGHT_PAREN,
  TOKEN_LEFT_BRACE,
  TOKEN_RIGHT_BRACE,
  TOKEN_SEMICOLON,
  /* Literals */
  TOKEN_IDENTIFIER,
  TOKEN_STRING,
  TOKEN_FLOAT,
  TOKEN_INT,
  TOKEN_UINT,
  /* TOKEN_ARRAY, */
  /* TOKEN_MAP, */
  TOKEN_FALSE,
  TOKEN_TRUE,
  TOKEN_EOF,
  TOKEN_ERROR,
  /* Keywords */
  TOKEN_ADD,
  TOKEN_SUB,
  TOKEN_MUL,
  TOKEN_DIV,
  TOKEN_MOD,
  TOKEN_GT,
  TOKEN_LT,
  TOKEN_EQ,
  TOKEN_GE,
  TOKEN_LE,
  TOKEN_NE,
  TOKEN_NULL,
  TOKEN_AND,
  TOKEN_OR,
  TOKEN_XOR,
  TOKEN_SHIFTRIGHT,
  TOKEN_SHIFTLEFT,
  TOKEN_PRINT,
} TokenType;

typedef struct Token Token;
struct Token
{
  TokenType type;
  char *start;
  int32_t length;
  int32_t line;
};

typedef struct SourceCode SourceCode;
struct SourceCode
{
  size_t size;
  char *buf;
  int count;
};

typedef struct TokenMap TokenMap;
struct TokenMap
{
  struct TokenMap* next;
  char* keyword;
  TokenType token;
};

unsigned int hash_Token(char* s);
TokenMap* lookup_Token(char* s);
TokenType get_Token(char* s);
TokenMap* put_Token(char* keyword, TokenType token);

void new_Tokenizer (char *src);
void new_TokenMap ();
void debug_printToken (Token t);
Token nextToken();

#endif
